@@ -3,9 +3,9 @@ import click
 import json
 import os
 import requests
-import requests
 from termcolor import cprint
 from pprint import pprint
+from requests.api import get, head
 from urllib3.exceptions import InsecureRequestWarning
 # Suppress only the single warning from urllib3 needed.
 requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
@@ -34,6 +34,168 @@ def check_env():
     print(f"Your environment has {octopus_server_uri} for the variable OCTOPUS_SERVER_URI")
 
 
+@main.command('get-spaces', short_help='Get a list of spaces for server')
+def get_spaces():
+    octopus_api_key = os.getenv('OCTOPUS_API_KEY')
+    octopus_server_uri = os.getenv('OCTOPUS_SERVER_URI')
+    headers = {'X-Octopus-ApiKey': octopus_api_key}
+    def get_octopus_resource(uri, headers, skip_count = 0):
+        """ Gets a resource from the API
+
+        Arguments:
+            uri: The base url of the Octopus Deploy API
+        """
+        items = []
+        response = requests.get((uri + "?skip=" + str(skip_count)), headers=headers, verify=False)
+        response.raise_for_status()
+        # Get results of API call
+        results = json.loads(response.content.decode('utf-8'))
+        # Store results
+        if 'Items' in results.keys():
+            items += results['Items']
+
+            # Check to see if there are more results
+            if (len(results['Items']) > 0) and (len(results['Items']) == results['ItemsPerPage']):
+                skip_count += results['ItemsPerPage']
+                items += get_octopus_resource(uri, headers, skip_count)
+        else:
+            return results
+        # return results
+        return items
+    uri = '{0}/spaces'.format(octopus_server_uri)
+    spaces = get_octopus_resource(uri, headers)
+    for i in spaces:
+        print(f"Space {i['Name']} has an ID of {i['Id']}")
+    return spaces
+
+
+@main.command('get-vars', short_help='Get a list of vars for a project in a space')
+@click.argument("space")
+@click.argument("project")
+def get_vars(space, project):
+    """ Get a list of vars for a project within a space
+
+    Arguments:
+        space: The space where the project resides
+        project: The project from which to get variables
+
+    """
+    def get_octopus_resource(uri, headers, skip_count = 0):
+        """ Gets a resource from the API
+
+        Arguments:
+            uri: The base url of the Octopus Deploy API
+        """
+        items = []
+        response = requests.get((uri + "?skip=" + str(skip_count)), headers=headers, verify=False)
+        response.raise_for_status()
+        # Get results of API call
+        results = json.loads(response.content.decode('utf-8'))
+        # Store results
+        if 'Items' in results.keys():
+            items += results['Items']
+
+            # Check to see if there are more results
+            if (len(results['Items']) > 0) and (len(results['Items']) == results['ItemsPerPage']):
+                skip_count += results['ItemsPerPage']
+                items += get_octopus_resource(uri, headers, skip_count)
+
+        else:
+            return results
+        # return results
+        return items
+    octopus_api_key = os.getenv('OCTOPUS_API_KEY')
+    octopus_server_uri = os.getenv('OCTOPUS_SERVER_URI')
+    headers = {'X-Octopus-ApiKey': octopus_api_key}
+    project_name = project
+    space_name = space
+    uri = '{0}/spaces'.format(octopus_server_uri)
+    spaces = get_octopus_resource(uri, headers)
+    space = next((x for x in spaces if x['Name'] == space_name), None)
+    uri = '{0}/{1}/projects'.format(octopus_server_uri, space['Id'])
+    projects = get_octopus_resource(uri, headers)
+    project = next((x for x in projects if x['Name'] == project_name), None)
+    if project != None:
+        uri = '{0}/{1}/variables/{2}'.format(octopus_server_uri, space['Id'], project['VariableSetId'])
+        project_variables = get_octopus_resource(uri, headers)
+        for var in project_variables['Variables']:
+            print(f"Found a variable with the ID of {var['Id']}, a name of {var['Name']}, and a value of {var['Value']}")
+        return project_variables
+    else:
+        return None
+
+
+@main.command('set-var', short_help='Set a variable for a project in a space')
+@click.argument("space")
+@click.argument("project")
+@click.argument("variable-name")
+@click.argument("variable-value")
+@click.option('--sensitive', is_flag=True)
+def set_var(space, project, variable_name, variable_value, sensitive):
+    """ Get a list of vars for a project within a space
+
+    Arguments:
+        space: The space where the project resides
+        project: The project from which to get variables
+
+    """
+    def get_octopus_resource(uri, headers, skip_count = 0):
+        """ Gets a resource from the API
+
+        Arguments:
+            uri: The base url of the Octopus Deploy API
+        """
+        items = []
+        response = requests.get((uri + "?skip=" + str(skip_count)), headers=headers, verify=False)
+        response.raise_for_status()
+        # Get results of API call
+        results = json.loads(response.content.decode('utf-8'))
+        # Store results
+        if 'Items' in results.keys():
+            items += results['Items']
+
+            # Check to see if there are more results
+            if (len(results['Items']) > 0) and (len(results['Items']) == results['ItemsPerPage']):
+                skip_count += results['ItemsPerPage']
+                items += get_octopus_resource(uri, headers, skip_count)
+
+        else:
+            return results
+        # return results
+        return items
+    octopus_api_key = os.getenv('OCTOPUS_API_KEY')
+    octopus_server_uri = os.getenv('OCTOPUS_SERVER_URI')
+    headers = {'X-Octopus-ApiKey': octopus_api_key}
+    project_name = project
+    space_name = space
+    variable = {
+            'Name': variable_name,
+            'Value': variable_value,
+            'Type': 'String',
+            'IsSensitive': sensitive
+    }
+    uri = '{0}/spaces'.format(octopus_server_uri)
+    spaces = get_octopus_resource(uri, headers)
+    space = next((x for x in spaces if x['Name'] == space_name), None)
+    uri = '{0}/{1}/projects'.format(octopus_server_uri, space['Id'])
+    projects = get_octopus_resource(uri, headers)
+    project = next((x for x in projects if x['Name'] == project_name), None)
+    if project != None:
+        uri = '{0}/{1}/variables/{2}'.format(octopus_server_uri, space['Id'], project['VariableSetId'])
+        project_variables = get_octopus_resource(uri, headers)
+        project_variable = next((x for x in project_variables['Variables'] if x['Name'] == variable['Name']), None)
+
+        if project_variable == None:
+            project_variables['Variables'].append(variable)
+        else:
+            project_variable['Value'] = variable['Value']
+            project_variable['Type'] = variable['Type']
+            project_variable['IsSensitive'] = variable ['IsSensitive']
+
+        response = requests.put(uri, headers=headers, json=project_variables, verify=False)
+        response.raise_for_status
+
+
 @main.command('get-projects', short_help='Get a list of projects for space')
 @click.argument("space")
 def get_projects(space):
@@ -41,6 +203,7 @@ def get_projects(space):
 
     Arguments:
         space: The name of the Octopus Deploy Space
+
     """
     octopus_api_key = os.getenv('OCTOPUS_API_KEY')
     octopus_server_uri = os.getenv('OCTOPUS_SERVER_URI')
